@@ -1,8 +1,8 @@
 # --
 # Kernel/Modules/CustomerTicketMessage.pm - to handle customer messages
-# Copyright (C) 2001-2011 OTRS AG, http://otrs.org/
+# Copyright (C) 2001-2012 OTRS AG, http://otrs.org/
 # --
-# $Id: CustomerTicketMessage.pm,v 1.83.2.1 2011/05/18 17:39:25 en Exp $
+# $Id: CustomerTicketMessage.pm,v 1.100.2.3 2012/07/01 12:29:48 ub Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -18,9 +18,12 @@ use Kernel::System::Web::UploadCache;
 use Kernel::System::SystemAddress;
 use Kernel::System::Queue;
 use Kernel::System::State;
+use Kernel::System::DynamicField;
+use Kernel::System::DynamicField::Backend;
+use Kernel::System::VariableCheck qw(:all);
 
 use vars qw($VERSION);
-$VERSION = qw($Revision: 1.83.2.1 $) [1];
+$VERSION = qw($Revision: 1.100.2.3 $) [1];
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -37,10 +40,12 @@ sub new {
     }
 
     # needed objects
-    $Self->{StateObject}       = Kernel::System::State->new(%Param);
-    $Self->{SystemAddress}     = Kernel::System::SystemAddress->new(%Param);
-    $Self->{QueueObject}       = Kernel::System::Queue->new(%Param);
-    $Self->{UploadCacheObject} = Kernel::System::Web::UploadCache->new(%Param);
+    $Self->{StateObject}        = Kernel::System::State->new(%Param);
+    $Self->{SystemAddress}      = Kernel::System::SystemAddress->new(%Param);
+    $Self->{QueueObject}        = Kernel::System::Queue->new(%Param);
+    $Self->{UploadCacheObject}  = Kernel::System::Web::UploadCache->new(%Param);
+    $Self->{DynamicFieldObject} = Kernel::System::DynamicField->new(%Param);
+    $Self->{BackendObject}      = Kernel::System::DynamicField::Backend->new(%Param);
 
     # get form id
     $Self->{FormID} = $Self->{ParamObject}->GetParam( Param => 'FormID' );
@@ -52,6 +57,13 @@ sub new {
 
     $Self->{Config} = $Self->{ConfigObject}->Get("Ticket::Frontend::$Self->{Action}");
 
+    # get the dynamic fields for this screen
+    $Self->{DynamicField} = $Self->{DynamicFieldObject}->DynamicFieldListGet(
+        Valid       => 1,
+        ObjectType  => [ 'Ticket', 'Article' ],
+        FieldFilter => $Self->{Config}->{DynamicField} || {},
+    );
+
     return $Self;
 }
 
@@ -64,154 +76,101 @@ sub Run {
         $GetParam{$Key} = $Self->{ParamObject}->GetParam( Param => $Key );
     }
 
-    # get article free text params
-    for my $Count ( 1 .. 3 ) {
-        my $Key  = 'ArticleFreeKey' . $Count;
-        my $Text = 'ArticleFreeText' . $Count;
-        $GetParam{$Key}  = $Self->{ParamObject}->GetParam( Param => $Key );
-        $GetParam{$Text} = $Self->{ParamObject}->GetParam( Param => $Text );
+    # get Dynamic fields from ParamObject
+    my %DynamicFieldValues;
+
+    # cycle trough the activated Dynamic Fields for this screen
+    DYNAMICFIELD:
+    for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+        next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
+
+        # extract the dynamic field value form the web request
+        $DynamicFieldValues{ $DynamicFieldConfig->{Name} } =
+            $Self->{BackendObject}->EditFieldValueGet(
+            DynamicFieldConfig => $DynamicFieldConfig,
+            ParamObject        => $Self->{ParamObject},
+            LayoutObject       => $Self->{LayoutObject},
+            );
     }
 
+    # convert dynamic field values into a structure for ACLs
+    my %DynamicFieldACLParameters;
+    DYNAMICFIELD:
+    for my $DynamicField ( keys %DynamicFieldValues ) {
+        next DYNAMICFIELD if !$DynamicField;
+        next DYNAMICFIELD if !$DynamicFieldValues{$DynamicField};
+
+        $DynamicFieldACLParameters{ 'DynamicField_' . $DynamicField }
+            = $DynamicFieldValues{$DynamicField};
+    }
+    $GetParam{DynamicField} = \%DynamicFieldACLParameters;
+
     if ( !$Self->{Subaction} ) {
 
-        #Get QueueID for services
+        #Get default Queue ID if none is set
         my $QueueDefaultID;
-        if ( !$GetParam{Dest} ) {
+        if ( !$GetParam{Dest} && !$Param{ToSelected} ) {
             my $QueueDefault = $Self->{Config}->{'QueueDefault'} || '';
             if ($QueueDefault) {
                 $QueueDefaultID = $Self->{QueueObject}->QueueLookup( Queue => $QueueDefault );
+                if ($QueueDefaultID) {
+                    $Param{ToSelected} = $QueueDefaultID . '||' . $QueueDefault;
+                }
             }
         }
 
-        # get default selections for ticket free text fields
-        my %TicketFreeDefault;
-        for my $Count ( 1 .. 16 ) {
-            my $Key  = 'TicketFreeKey' . $Count;
-            my $Text = 'TicketFreeText' . $Count;
-            $TicketFreeDefault{$Key}  = $Self->{ConfigObject}->Get( $Key . '::DefaultSelection' );
-            $TicketFreeDefault{$Text} = $Self->{ConfigObject}->Get( $Text . '::DefaultSelection' );
-        }
+        # create html strings for all dynamic fields
+        my %DynamicFieldHTML;
 
-        # get free text config options
-        my %TicketFreeText;
-        for my $Count ( 1 .. 16 ) {
-            my $Key  = 'TicketFreeKey' . $Count;
-            my $Text = 'TicketFreeText' . $Count;
-            $TicketFreeText{$Key} = $Self->{TicketObject}->TicketFreeTextGet(
-                TicketID       => $Self->{TicketID},
-                Action         => $Self->{Action},
-                Type           => $Key,
-                CustomerUserID => $Self->{UserID},
-            );
-            $TicketFreeText{$Text} = $Self->{TicketObject}->TicketFreeTextGet(
-                TicketID       => $Self->{TicketID},
-                Action         => $Self->{Action},
-                Type           => $Text,
-                CustomerUserID => $Self->{UserID},
-            );
+        # cycle trough the activated Dynamic Fields for this screen
+        DYNAMICFIELD:
+        for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+            next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
-            # If Key has value 2, this means that the freetextfield is required
-            if (
-                $Self->{Config}->{TicketFreeText}->{$Count}
-                && $Self->{Config}->{TicketFreeText}->{$Count} == 2
-                )
-            {
-                $TicketFreeText{Required}->{$Count} = 1;
-            }
-        }
+            my $PossibleValuesFilter;
 
-        my %TicketFreeTextHTML = $Self->{LayoutObject}->AgentFreeText(
-            Config => \%TicketFreeText,
-            Ticket => \%TicketFreeDefault,
-        );
+            # check if field has PossibleValues property in its configuration
+            if ( IsHashRefWithData( $DynamicFieldConfig->{Config}->{PossibleValues} ) ) {
 
-        # get ticket free time params
-        my %TicketFreeTime;
-        for my $Count ( 1 .. 6 ) {
-            for my $Type (qw(Used Year Month Day Hour Minute)) {
-                $TicketFreeTime{ "TicketFreeTime" . $Count . $Type }
-                    = $Self->{ParamObject}->GetParam( Param => "TicketFreeTime" . $Count . $Type );
-            }
-            $TicketFreeTime{ 'TicketFreeTime' . $Count . 'Optional' }
-                = $Self->{ConfigObject}->Get( 'TicketFreeTimeOptional' . $Count ) || 0;
-            if ( !$Self->{ConfigObject}->Get( 'TicketFreeTimeOptional' . $Count ) ) {
-                $TicketFreeTime{ 'TicketFreeTime' . $Count . 'Used' } = 1;
+                # set possible values filter from ACLs
+                my $ACL = $Self->{TicketObject}->TicketAcl(
+                    %GetParam,
+                    Action         => $Self->{Action},
+                    TicketID       => $Self->{TicketID},
+                    ReturnType     => 'Ticket',
+                    ReturnSubType  => 'DynamicField_' . $DynamicFieldConfig->{Name},
+                    Data           => $DynamicFieldConfig->{Config}->{PossibleValues},
+                    CustomerUserID => $Self->{UserID},
+                );
+                if ($ACL) {
+                    my %Filter = $Self->{TicketObject}->TicketAclData();
+                    $PossibleValuesFilter = \%Filter;
+                }
             }
 
-            if ( $Self->{Config}->{TicketFreeTime}->{$Count} == 2 ) {
-                $TicketFreeTime{ 'TicketFreeTime' . $Count . 'Required' } = 1;
-            }
+            # get field html
+            $DynamicFieldHTML{ $DynamicFieldConfig->{Name} } =
+                $Self->{BackendObject}->EditFieldRender(
+                DynamicFieldConfig   => $DynamicFieldConfig,
+                PossibleValuesFilter => $PossibleValuesFilter,
+                Mandatory =>
+                    $Self->{Config}->{DynamicField}->{ $DynamicFieldConfig->{Name} } == 2,
+                LayoutObject => $Self->{LayoutObject},
+                ParamObject  => $Self->{ParamObject},
+
+                # CustomerTicketMessage does not support AJAXUpdate
+                AJAXUpdate     => 0,
+                SubmitOnChange => 1,
+                );
         }
 
-        # transform free time, time stamp based on user time zone
-        for my $Count ( 1 .. 6 ) {
-            my $Prefix = 'TicketFreeTime' . $Count;
-            next if !$TicketFreeTime{ $Prefix . 'Year' };
-            next if !$TicketFreeTime{ $Prefix . 'Month' };
-            next if !$TicketFreeTime{ $Prefix . 'Day' };
-            next if !$TicketFreeTime{ $Prefix . 'Hour' };
-            next if !$TicketFreeTime{ $Prefix . 'Minute' };
-            %TicketFreeTime = $Self->{LayoutObject}->TransfromDateSelection(
-                %TicketFreeTime,
-                Prefix => $Prefix
-            );
-        }
-
-        # free time
-        my %FreeTime = $Self->{LayoutObject}->CustomerFreeDate(
-            %Param,
-            Ticket => \%TicketFreeTime,
-        );
-
-        # get default selections for article free text fields
-        my %ArticleFreeDefault;
-        for my $Count ( 1 .. 3 ) {
-            my $Key  = 'ArticleFreeKey' . $Count;
-            my $Text = 'ArticleFreeText' . $Count;
-            $ArticleFreeDefault{$Key}  = $Self->{ConfigObject}->Get( $Key . '::DefaultSelection' );
-            $ArticleFreeDefault{$Text} = $Self->{ConfigObject}->Get( $Text . '::DefaultSelection' );
-        }
-
-        # get article free text config options
-        my %ArticleFreeText;
-        for my $Count ( 1 .. 3 ) {
-            my $Key  = 'ArticleFreeKey' . $Count;
-            my $Text = 'ArticleFreeText' . $Count;
-            $ArticleFreeText{$Key} = $Self->{TicketObject}->ArticleFreeTextGet(
-                ArticleID      => $Self->{ArticleID},
-                Action         => $Self->{Action},
-                Type           => $Key,
-                CustomerUserID => $Self->{UserID},
-            );
-            $ArticleFreeText{$Text} = $Self->{TicketObject}->ArticleFreeTextGet(
-                TicketID       => $Self->{TicketID},
-                Action         => $Self->{Action},
-                Type           => $Text,
-                CustomerUserID => $Self->{UserID},
-            );
-
-            # If Key has value 2, this means that the freetextfield is required
-            if ( $Self->{Config}->{ArticleFreeText}->{$Count} == 2 ) {
-                $ArticleFreeText{Required}->{$Count} = 1;
-            }
-        }
-
-        my %ArticleFreeTextHTML = $Self->{LayoutObject}->TicketArticleFreeText(
-            Config  => \%ArticleFreeText,
-            Article => {
-                %GetParam,
-                %ArticleFreeDefault,
-            },
-        );
-
         # print form ...
         my $Output .= $Self->{LayoutObject}->CustomerHeader();
         $Output    .= $Self->{LayoutObject}->CustomerNavigationBar();
         $Output    .= $Self->_MaskNew(
-            QueueID => $QueueDefaultID,
-            %TicketFreeTextHTML,
-            %FreeTime,
-            %ArticleFreeTextHTML,
+            QueueID          => $QueueDefaultID,
+            ToSelected       => $Param{ToSelected},
+            DynamicFieldHTML => \%DynamicFieldHTML,
         );
         $Output .= $Self->{LayoutObject}->CustomerFooter();
         return $Output;
@@ -275,124 +234,83 @@ sub Run {
             FormID => $Self->{FormID},
         );
 
-        my %TicketFree;
-        for my $Count ( 1 .. 16 ) {
-            my $Key  = 'TicketFreeKey' . $Count;
-            my $Text = 'TicketFreeText' . $Count;
-            $TicketFree{$Key}  = $Self->{ParamObject}->GetParam( Param => $Key );
-            $TicketFree{$Text} = $Self->{ParamObject}->GetParam( Param => $Text );
-        }
+        # create html strings for all dynamic fields
+        my %DynamicFieldHTML;
 
-        # get free text config options
-        my %TicketFreeText;
-        for my $Count ( 1 .. 16 ) {
-            my $Key  = 'TicketFreeKey' . $Count;
-            my $Text = 'TicketFreeText' . $Count;
-            $TicketFreeText{$Key} = $Self->{TicketObject}->TicketFreeTextGet(
-                TicketID       => $Self->{TicketID},
-                Action         => $Self->{Action},
-                Type           => $Key,
-                QueueID        => $NewQueueID || 0,
-                ServiceID      => $GetParam{ServiceID} || 0,
-                CustomerUserID => $Self->{UserID},
-            );
-            $TicketFreeText{$Text} = $Self->{TicketObject}->TicketFreeTextGet(
-                TicketID       => $Self->{TicketID},
-                Action         => $Self->{Action},
-                Type           => $Text,
-                QueueID        => $NewQueueID || 0,
-                ServiceID      => $GetParam{ServiceID} || 0,
-                CustomerUserID => $Self->{UserID},
-            );
+        # cycle trough the activated Dynamic Fields for this screen
+        DYNAMICFIELD:
+        for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+            next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
-            # If Key has value 2, this means that the freetextfield is required
-            if ( $Self->{Config}->{TicketFreeText}->{$Count} == 2 ) {
-                $TicketFreeText{Required}->{$Count} = 1;
+            my $PossibleValuesFilter;
+
+            # check if field has PossibleValues property in its configuration
+            if ( IsHashRefWithData( $DynamicFieldConfig->{Config}->{PossibleValues} ) ) {
+
+                # set possible values filter from ACLs
+                my $ACL = $Self->{TicketObject}->TicketAcl(
+                    %GetParam,
+                    Action         => $Self->{Action},
+                    TicketID       => $Self->{TicketID},
+                    ReturnType     => 'Ticket',
+                    ReturnSubType  => 'DynamicField_' . $DynamicFieldConfig->{Name},
+                    Data           => $DynamicFieldConfig->{Config}->{PossibleValues},
+                    CustomerUserID => $Self->{UserID},
+                );
+                if ($ACL) {
+                    my %Filter = $Self->{TicketObject}->TicketAclData();
+                    $PossibleValuesFilter = \%Filter;
+                }
             }
 
-            # check required FreeTextField (if configured)
-            if (
-                $Self->{Config}->{TicketFreeText}->{$Count} == 2
-                && $TicketFree{$Text} eq ''
-                && !$GetParam{Expand}
-                && $IsUpload == 0
-                )
-            {
-                $TicketFreeText{Error}->{$Count} = 1;
-                $Error{$Text} = 'ServerError';
-            }
-        }
-        my %TicketFreeTextHTML = $Self->{LayoutObject}->AgentFreeText(
-            Config => \%TicketFreeText,
-            Ticket => \%TicketFree,
-        );
+            my $ValidationResult;
 
-        # get ticket free time params
-        my %TicketFreeTime;
-        for my $Count ( 1 .. 6 ) {
-            for my $Type (qw(Used Year Month Day Hour Minute)) {
-                $TicketFreeTime{ "TicketFreeTime" . $Count . $Type }
-                    = $Self->{ParamObject}->GetParam(
-                    Param => "TicketFreeTime" . $Count . $Type,
+            # do not validate on attachment upload or GetParam Expand
+            if ( !$IsUpload && !$GetParam{Expand} ) {
+
+                $ValidationResult = $Self->{BackendObject}->EditFieldValueValidate(
+                    DynamicFieldConfig   => $DynamicFieldConfig,
+                    PossibleValuesFilter => $PossibleValuesFilter,
+                    ParamObject          => $Self->{ParamObject},
+                    Mandatory =>
+                        $Self->{Config}->{DynamicField}->{ $DynamicFieldConfig->{Name} } == 2,
+                );
+
+                if ( !IsHashRefWithData($ValidationResult) ) {
+                    my $Output = $Self->{LayoutObject}->CustomerHeader( Title => 'Error' );
+                    $Output .= $Self->{LayoutObject}->CustomerError(
+                        Message =>
+                            "Could not perform validation on field $DynamicFieldConfig->{Label}!",
+                        Comment => 'Please contact your administrator',
                     );
-            }
-            $TicketFreeTime{ 'TicketFreeTime' . $Count . 'Optional' }
-                = $Self->{ConfigObject}->Get( 'TicketFreeTimeOptional' . $Count ) || 0;
-            if ( !$Self->{ConfigObject}->Get( 'TicketFreeTimeOptional' . $Count ) ) {
-                $TicketFreeTime{ 'TicketFreeTime' . $Count . 'Used' } = 1;
+                    $Output .= $Self->{LayoutObject}->CustomerFooter();
+                    return $Output;
+                }
+
+                # propagate validation error to the Error variable to be detected by the frontend
+                if ( $ValidationResult->{ServerError} ) {
+                    $Error{ $DynamicFieldConfig->{Name} } = ' ServerError';
+                }
             }
 
-            if ( $Self->{Config}->{TicketFreeTime}->{$Count} == 2 ) {
-                $TicketFreeTime{ 'TicketFreeTime' . $Count . 'Required' } = 1;
-            }
+            # get field html
+            $DynamicFieldHTML{ $DynamicFieldConfig->{Name} } =
+                $Self->{BackendObject}->EditFieldRender(
+                DynamicFieldConfig   => $DynamicFieldConfig,
+                PossibleValuesFilter => $PossibleValuesFilter,
+                Mandatory =>
+                    $Self->{Config}->{DynamicField}->{ $DynamicFieldConfig->{Name} } == 2,
+                ServerError  => $ValidationResult->{ServerError}  || '',
+                ErrorMessage => $ValidationResult->{ErrorMessage} || '',
+                LayoutObject => $Self->{LayoutObject},
+                ParamObject  => $Self->{ParamObject},
+
+                # CustomerTicketMessage does not support AJAXUpdate
+                AJAXUpdate     => 0,
+                SubmitOnChange => 1,
+                );
         }
 
-        # free time
-        my %FreeTime = $Self->{LayoutObject}->CustomerFreeDate(
-            %Param,
-            Ticket => \%TicketFreeTime,
-        );
-
-        # article free text
-        my %ArticleFreeText;
-        for my $Count ( 1 .. 3 ) {
-            my $Key  = 'ArticleFreeKey' . $Count;
-            my $Text = 'ArticleFreeText' . $Count;
-            $ArticleFreeText{$Key} = $Self->{TicketObject}->ArticleFreeTextGet(
-                TicketID       => $Self->{TicketID},
-                Type           => $Key,
-                Action         => $Self->{Action},
-                CustomerUserID => $Self->{UserID},
-            );
-            $ArticleFreeText{$Text} = $Self->{TicketObject}->ArticleFreeTextGet(
-                TicketID       => $Self->{TicketID},
-                Type           => $Text,
-                Action         => $Self->{Action},
-                CustomerUserID => $Self->{UserID},
-            );
-
-            # If Key has value 2, this means that the field is required
-            if ( $Self->{Config}->{ArticleFreeText}->{$Count} == 2 ) {
-                $ArticleFreeText{Required}->{$Count} = 1;
-            }
-
-            # check required ArticleTextField (if configured)
-            if (
-                $Self->{Config}->{ArticleFreeText}->{$Count} == 2
-                && $GetParam{$Text} eq ''
-                && !$GetParam{Expand}
-                && $IsUpload == 0
-                )
-            {
-                $ArticleFreeText{Error}->{$Count} = 1;
-                $Error{$Text} = 'ServerError';
-            }
-        }
-        my %ArticleFreeTextHTML = $Self->{LayoutObject}->TicketArticleFreeText(
-            Config  => \%ArticleFreeText,
-            Article => \%GetParam,
-        );
-
         # rewrap body if rich text is used
         if ( $Self->{LayoutObject}->{BrowserRichText} && $GetParam{Body} ) {
             $GetParam{Body}
@@ -437,18 +355,16 @@ sub Run {
             $Output    .= $Self->_MaskNew(
                 Attachments => \@Attachments,
                 %GetParam,
-                ToSelected => $Dest,
-                QueueID    => $NewQueueID,
-                %TicketFreeTextHTML,
-                %FreeTime,
-                %ArticleFreeTextHTML,
-                Errors => \%Error,
+                ToSelected       => $Dest,
+                QueueID          => $NewQueueID,
+                DynamicFieldHTML => \%DynamicFieldHTML,
+                Errors           => \%Error,
             );
             $Output .= $Self->{LayoutObject}->CustomerFooter();
             return $Output;
         }
 
-        # if customer is not alowed to set priority, set it to default
+        # if customer is not allowed to set priority, set it to default
         if ( !$Self->{Config}->{Priority} ) {
             $GetParam{PriorityID} = '';
             $GetParam{Priority}   = $Self->{Config}->{PriorityDefault};
@@ -471,46 +387,19 @@ sub Run {
             UserID       => $Self->{ConfigObject}->Get('CustomerPanelUserID'),
         );
 
-        # set ticket free text
-        for my $Count ( 1 .. 16 ) {
-            my $Key  = 'TicketFreeKey' . $Count;
-            my $Text = 'TicketFreeText' . $Count;
-            if ( defined $TicketFree{$Key} ) {
-                $Self->{TicketObject}->TicketFreeTextSet(
-                    TicketID => $TicketID,
-                    Key      => $TicketFree{$Key},
-                    Value    => $TicketFree{$Text},
-                    Counter  => $Count,
-                    UserID   => $Self->{ConfigObject}->Get('CustomerPanelUserID'),
-                );
-            }
-        }
+        # set ticket dynamic fields
+        # cycle trough the activated Dynamic Fields for this screen
+        DYNAMICFIELD:
+        for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+            next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
+            next DYNAMICFIELD if $DynamicFieldConfig->{ObjectType} ne 'Ticket';
 
-        # set ticket free time
-        for my $Count ( 1 .. 6 ) {
-            my $Prefix = 'TicketFreeTime' . $Count;
-            next if !defined $TicketFreeTime{ $Prefix . 'Year' };
-            next if !defined $TicketFreeTime{ $Prefix . 'Month' };
-            next if !defined $TicketFreeTime{ $Prefix . 'Day' };
-            next if !defined $TicketFreeTime{ $Prefix . 'Hour' };
-            next if !defined $TicketFreeTime{ $Prefix . 'Minute' };
-
-            # set time stamp to NULL if field is not used/checked
-            if ( !$TicketFreeTime{ $Prefix . 'Used' } ) {
-                $TicketFreeTime{ $Prefix . 'Year' }   = 0;
-                $TicketFreeTime{ $Prefix . 'Month' }  = 0;
-                $TicketFreeTime{ $Prefix . 'Day' }    = 0;
-                $TicketFreeTime{ $Prefix . 'Hour' }   = 0;
-                $TicketFreeTime{ $Prefix . 'Minute' } = 0;
-            }
-
-            # set free time
-            $Self->{TicketObject}->TicketFreeTimeSet(
-                %TicketFreeTime,
-                Prefix   => 'TicketFreeTime',
-                TicketID => $TicketID,
-                Counter  => $Count,
-                UserID   => $Self->{ConfigObject}->Get('CustomerPanelUserID'),
+            # set the value
+            my $Success = $Self->{BackendObject}->ValueSet(
+                DynamicFieldConfig => $DynamicFieldConfig,
+                ObjectID           => $TicketID,
+                Value              => $DynamicFieldValues{ $DynamicFieldConfig->{Name} },
+                UserID             => $Self->{ConfigObject}->Get('CustomerPanelUserID'),
             );
         }
 
@@ -555,20 +444,20 @@ sub Run {
             return $Output;
         }
 
-        # set article free text
-        for my $Count ( 1 .. 3 ) {
-            my $Key  = 'ArticleFreeKey' . $Count;
-            my $Text = 'ArticleFreeText' . $Count;
-            if ( defined $GetParam{$Key} ) {
-                $Self->{TicketObject}->ArticleFreeTextSet(
-                    TicketID  => $TicketID,
-                    ArticleID => $ArticleID,
-                    Key       => $GetParam{$Key},
-                    Value     => $GetParam{$Text},
-                    Counter   => $Count,
-                    UserID    => $Self->{ConfigObject}->Get('CustomerPanelUserID'),
-                );
-            }
+        # set article dynamic fields
+        # cycle trough the activated Dynamic Fields for this screen
+        DYNAMICFIELD:
+        for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+            next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
+            next DYNAMICFIELD if $DynamicFieldConfig->{ObjectType} ne 'Article';
+
+            # set the value
+            my $Success = $Self->{BackendObject}->ValueSet(
+                DynamicFieldConfig => $DynamicFieldConfig,
+                ObjectID           => $ArticleID,
+                Value              => $DynamicFieldValues{ $DynamicFieldConfig->{Name} },
+                UserID             => $Self->{ConfigObject}->Get('CustomerPanelUserID'),
+            );
         }
 
         # get pre loaded attachment
@@ -656,7 +545,7 @@ sub _MaskNew {
                     Message  => "Module: $Module loaded!",
                 );
             }
-            %NewTos = ( $Object->Run( Env => $Self ), ( '', => '-' ) );
+            %NewTos = ( $Object->Run( Env => $Self, ACLParams => \%Param ), ( '', => '-' ) );
         }
         else {
             return $Self->{LayoutObject}->FatalError();
@@ -675,7 +564,7 @@ sub _MaskNew {
             Size       => 0,
             Name       => 'Dest',
             Class      => "Validate_Required " . $Param{Errors}->{QueueInvalid},
-            SelectedID => $Param{ToSelected},
+            SelectedID => $Param{ToSelected} || $Param{QueueID},
         );
         $Self->{LayoutObject}->Block(
             Name => 'Queue',
@@ -690,6 +579,7 @@ sub _MaskNew {
     # get priority
     if ( $Self->{Config}->{Priority} ) {
         my %Priorities = $Self->{TicketObject}->TicketPriorityList(
+            %Param,
             CustomerUserID => $Self->{UserID},
             Action         => $Self->{Action},
         );
@@ -740,16 +630,16 @@ sub _MaskNew {
 
     # services
     if ( $Self->{ConfigObject}->Get('Ticket::Service') && $Self->{Config}->{Service} ) {
-        my %Service;
+        my %Services;
         if ( $Param{QueueID} || $Param{TicketID} ) {
-            %Service = $Self->{TicketObject}->TicketServiceList(
+            %Services = $Self->{TicketObject}->TicketServiceList(
                 %Param,
                 Action         => $Self->{Action},
                 CustomerUserID => $Self->{UserID},
             );
         }
         $Param{ServiceStrg} = $Self->{LayoutObject}->BuildSelection(
-            Data         => \%Service,
+            Data         => \%Services,
             Name         => 'ServiceID',
             SelectedID   => $Param{ServiceID},
             PossibleNone => 1,
@@ -762,6 +652,11 @@ sub _MaskNew {
             Name => 'TicketService',
             Data => \%Param,
         );
+
+        # reset previous ServiceID to reset SLA-List if no service is selected
+        if ( !$Services{ $Param{ServiceID} || '' } ) {
+            $Param{ServiceID} = '';
+        }
         my %SLA;
         if ( $Self->{Config}->{SLA} ) {
             if ( $Param{ServiceID} ) {
@@ -794,45 +689,36 @@ sub _MaskNew {
         }
     }
 
-    # ticket free text
-    for my $Count ( 1 .. 16 ) {
-        next if !$Self->{Config}->{TicketFreeText}->{$Count};
+    # Dynamic fields
+    # cycle trough the activated Dynamic Fields for this screen
+    DYNAMICFIELD:
+    for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+        next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
+
+        # skip fields that HTML could not be retrieved
+        next DYNAMICFIELD if !IsHashRefWithData(
+            $Param{DynamicFieldHTML}->{ $DynamicFieldConfig->{Name} }
+        );
+
+        # get the html strings form $Param
+        my $DynamicFieldHTML = $Param{DynamicFieldHTML}->{ $DynamicFieldConfig->{Name} };
+
         $Self->{LayoutObject}->Block(
-            Name => 'FreeText',
+            Name => 'DynamicField',
             Data => {
-                TicketFreeKeyField  => $Param{ 'TicketFreeKeyField' . $Count },
-                TicketFreeTextField => $Param{ 'TicketFreeTextField' . $Count },
-                Count               => $Count,
-                ServerError         => $Param{Errors}->{ 'TicketFreeText' . $Count },
-                %Param,
+                Name  => $DynamicFieldConfig->{Name},
+                Label => $DynamicFieldHTML->{Label},
+                Field => $DynamicFieldHTML->{Field},
             },
         );
-    }
 
-    # ticket free time
-    for my $Count ( 1 .. 6 ) {
-        next if !$Self->{Config}->{TicketFreeTime}->{$Count};
+        # example of dynamic fields order customization
         $Self->{LayoutObject}->Block(
-            Name => 'FreeTime',
+            Name => 'DynamicField_' . $DynamicFieldConfig->{Name},
             Data => {
-                TicketFreeTimeKey => $Param{ 'TicketFreeTimeKey' . $Count },
-                TicketFreeTime    => $Param{ 'TicketFreeTime' . $Count },
-                Count             => $Count,
-            },
-        );
-    }
-
-    # article free text
-    for my $Count ( 1 .. 3 ) {
-        next if !$Self->{Config}->{ArticleFreeText}->{$Count};
-        $Self->{LayoutObject}->Block(
-            Name => 'ArticleFreeText',
-            Data => {
-                ArticleFreeKeyField  => $Param{ 'ArticleFreeKeyField' . $Count },
-                ArticleFreeTextField => $Param{ 'ArticleFreeTextField' . $Count },
-                Count                => $Count,
-                ServerError          => $Param{Errors}->{ 'ArticletFreeText' . $Count },
-                %Param,
+                Name  => $DynamicFieldConfig->{Name},
+                Label => $DynamicFieldHTML->{Label},
+                Field => $DynamicFieldHTML->{Field},
             },
         );
     }
@@ -846,31 +732,6 @@ sub _MaskNew {
         );
     }
 
-    # java script check for required free text fields by form submit
-    for my $Key ( keys %{ $Self->{Config}->{TicketFreeText} } ) {
-        next if $Self->{Config}->{TicketFreeText}->{$Key} != 2;
-        $Self->{LayoutObject}->Block(
-            Name => 'TicketFreeTextCheckJs',
-            Data => {
-                TicketFreeTextField => "TicketFreeText$Key",
-                TicketFreeKeyField  => "TicketFreeKey$Key",
-            },
-        );
-    }
-
-   #    # java script check for required free time fields by form submit
-   #    for my $Key ( keys %{ $Self->{Config}->{TicketFreeTime} } ) {
-   #        next if $Self->{Config}->{TicketFreeTime}->{$Key} != 2;
-   #        $Self->{LayoutObject}->Block(
-   #            Name => 'TicketFreeTimeCheckJs',
-   #            Data => {
-   #                TicketFreeTimeCheck => 'TicketFreeTime' . $Key . 'Used',
-   #                TicketFreeTimeField => 'TicketFreeTime' . $Key,
-   #                TicketFreeTimeKey   => $Self->{ConfigObject}->Get( 'TicketFreeTimeKey' . $Key ),
-   #            },
-   #        );
-   #    }
-
     # add rich text editor
     if ( $Self->{LayoutObject}->{BrowserRichText} ) {
         $Self->{LayoutObject}->Block(
